package main

import (
	"crypto/rand"
	"crypto/rsa"
	"crypto/x509"
	"encoding/pem"
	"testing"

	"golang.org/x/crypto/ssh"
)

func privRSAKey(t *testing.T, bitSize int) *rsa.PrivateKey {
	t.Helper()
	k, err := rsa.GenerateKey(rand.Reader, bitSize)
	if err != nil {
		t.Fatalf("Could not generate RSA key for testing: %v", err)
	}

	if err := k.Validate(); err != nil {
		t.Fatalf("Could not validate RSA key: %v", err)
	}

	return k
}

func privRSAPEM(t *testing.T, k *rsa.PrivateKey) []byte {
	t.Helper()
	der := x509.MarshalPKCS1PrivateKey(k)
	return pem.EncodeToMemory(&pem.Block{
		Type:  "RSA PRIVATE KEY",
		Bytes: der,
	})
}

func pubRSAKey(t *testing.T, priv *rsa.PrivateKey) []byte {
	t.Helper()
	k, err := ssh.NewPublicKey(&priv.PublicKey)
	if err != nil {
		t.Fatalf("Could not generate public key: %v", err)
	}
	return ssh.MarshalAuthorizedKey(k)
}
