package key_test

import (
	"crypto/rand"
	"crypto/rsa"
	"crypto/x509"
	"encoding/pem"
	"testing"

	"golang.org/x/crypto/ssh"
)

func privRSAKey(t *testing.T, bitSize int) *rsa.PrivateKey {
	t.Helper()
	k, err := rsa.GenerateKey(rand.Reader, bitSize)
	if err != nil {
		t.Fatalf("Could not generate RSA key for testing: %v", err)
	}

	if err := k.Validate(); err != nil {
		t.Fatalf("Could not validate RSA key: %v", err)
	}

	return k
}

func privRSAPEM(t *testing.T, k *rsa.PrivateKey, password []byte) []byte {
	t.Helper()
	der := x509.MarshalPKCS1PrivateKey(k)
	block := &pem.Block{
		Type:  "RSA PRIVATE KEY",
		Bytes: der,
	}
	if password != nil {
		encrypted, err := x509.EncryptPEMBlock(rand.Reader, "RSA PRIVATE KEY", der, password, x509.PEMCipherAES128)
		if err != nil {
			t.Fatalf("Could not encrypt RSA private key: %v", err)
		}
		block = encrypted
	}
	return pem.EncodeToMemory(block)
}

func pubRSAKey(t *testing.T, priv *rsa.PrivateKey) []byte {
	t.Helper()
	k, err := ssh.NewPublicKey(&priv.PublicKey)
	if err != nil {
		t.Fatalf("Could not generate public key: %v", err)
	}
	return ssh.MarshalAuthorizedKey(k)
}
