package main

import (
	"fmt"
	"os"

	"git.sr.ht/~minus/sshign/key"
	"golang.org/x/crypto/ssh/terminal"
)

var version = "unknown"
var buildDate = "unknown"

func main() {
	switch len(os.Args) {
	case 2:
		f, err := os.Open(os.Args[1])
		if err != nil {
			panic(fmt.Sprintf("%s: could not open file %s: %v", os.Args[0], os.Args[1], err))
		}
		defer f.Close()

		sig, err := key.Sign(ttyRequester{}, f, os.Stdin)
		if err != nil {
			panic(fmt.Sprintf("%s: %v", os.Args[0], err))
		}
		fmt.Printf("%s\n", sig)
	case 3:
		pubKey, err := os.Open(os.Args[1])
		if err != nil {
			panic(fmt.Sprintf("%s: could not open public key %s: %v", os.Args[0], os.Args[1], err))
		}
		defer pubKey.Close()

		sig, err := os.Open(os.Args[2])
		if err != nil {
			panic(fmt.Sprintf("%s: could not open signature file %s: %v", os.Args[0], os.Args[2], err))
		}
		defer sig.Close()

		switch err := key.Verify(pubKey, sig, os.Stdin).(type) {
		case nil:
			fmt.Println("verify: ok")
		case key.VerifyError:
			fmt.Printf("verify: %s\n", err)
			os.Exit(1)
		default:
			panic(fmt.Sprintf("%s: %v", os.Args[0], err))
		}
	default:
		printUsage(os.Args[0])
		os.Exit(1)
	}
}

func printUsage(called string) {
	fmt.Printf("sshign %s (%s)\n", version, buildDate)
	fmt.Printf("usage:\n")
	fmt.Printf("  signing: %s <secret key file> < data-to-sign > signature\n", called)
	fmt.Printf("  verify:  %s <public key file> <signature file> < data-to-verify\n", called)
}

type ttyRequester struct {
}

func (r ttyRequester) Passphrase(path string) ([]byte, error) {
	out, err := os.OpenFile("/dev/tty", os.O_WRONLY, 0)
	if err != nil {
		return nil, fmt.Errorf("unable to open TTY to request password: %w", err)
	}
	defer out.Close()

	in, err := os.Open("/dev/tty")
	if err != nil {
		return nil, fmt.Errorf("unable to open TTY to read password: %w", err)
	}
	defer in.Close()

	if _, err := fmt.Fprintf(out, "Enter passphrase for %s: ", path); err != nil {
		return nil, fmt.Errorf("unable to request passphrase from tty: %w", err)
	}

	pass, err := terminal.ReadPassword(int(in.Fd()))
	if err != nil {
		return nil, fmt.Errorf("unable to read passphrase from tty: %w", err)
	}
	if _, err := fmt.Fprint(out, "\n"); err != nil {
		return nil, fmt.Errorf("unable to print newline to tty: %w", err)
	}
	return pass, nil
}
