package main_test

import (
	"bytes"
	"flag"
	"fmt"
	"io/ioutil"
	"log"
	"os"
	"os/exec"
	"testing"

	"gotest.tools/v3/golden"
)

const binPath = "./sshign"

var update = flag.Bool("update", false, "update golden files")

func goldenUpdate(t *testing.T, path string, body []byte) {
	t.Helper()
	fullPath := golden.Path(path)
	if err := ioutil.WriteFile(fullPath, body, 0644); err != nil {
		t.Fatalf("goldenCreate: could not update %s: %s", fullPath, err)
	}
}

func TestMain(m *testing.M) {
	os.Exit(testMain(m))
}

// testMain provides the TestMain functionality, but returns
// the exit code rather than directly calling os.Exit.  This
// is important because os.Exit causes the program to exit
// immediately, before any deferred function calls are run.
// Using this helper function allows us to still rely on
// defer statements for test teardown.
func testMain(m *testing.M) int {
	if out, err := exec.Command("go", "build", "-o", binPath).CombinedOutput(); err != nil {
		fmt.Println("Could not build!")
		fmt.Println("  Build Output:")
		fmt.Println(string(out))
		return 1
	}

	defer func() {
		if err := os.Remove(binPath); err != nil {
			log.Printf("Warning: could not clean up; %s left behind.", binPath)
		}
	}()

	return m.Run()
}

func TestSign(t *testing.T) {
	msg := golden.Open(t, "message")
	defer msg.Close()

	var out bytes.Buffer

	cmd := exec.Command(binPath, golden.Path("id_rsa"))
	cmd.Stdin = msg
	cmd.Stdout = &out
	cmd.Stderr = os.Stderr
	if err := cmd.Run(); err != nil {
		t.Fatalf("Could not run sign command: %s", err)
	}

	if *update {
		goldenUpdate(t, "sign.golden", out.Bytes())
	}

	golden.Assert(t, out.String(), "sign.golden")
}

func TestVerify(t *testing.T) {
	msg := golden.Open(t, "message")
	defer msg.Close()

	var out bytes.Buffer

	cmd := exec.Command(binPath, golden.Path("id_rsa.pub"), golden.Path("sign.golden"))
	cmd.Stdin = msg
	cmd.Stdout = &out
	cmd.Stderr = os.Stderr
	if err := cmd.Run(); err != nil {
		t.Fatalf("Could not run verify command: %s", err)
	}

	if *update {
		goldenUpdate(t, "verify.golden", out.Bytes())
	}

	golden.Assert(t, out.String(), "verify.golden")
}
